print("import: 'jupyterlab_plotly'")
import jupyterlab_plotly

print("import: 'plotly'")
import plotly

print("import: 'plotly.offline'")
import plotly.offline

print("import: 'plotly.io'")
import plotly.io

print("import: 'plotly.matplotlylib'")
import plotly.matplotlylib

print("import: 'plotly.matplotlylib.mplexporter'")
import plotly.matplotlylib.mplexporter

print("import: 'plotly.matplotlylib.mplexporter.renderers'")
import plotly.matplotlylib.mplexporter.renderers

print("import: 'plotly.figure_factory'")
import plotly.figure_factory

print("import: 'plotly.data'")
import plotly.data

print("import: 'plotly.colors'")
import plotly.colors

print("import: 'plotly.express'")
import plotly.express

print("import: 'plotly.express.data'")
import plotly.express.data

print("import: 'plotly.express.colors'")
import plotly.express.colors

print("import: 'plotly.express.trendline_functions'")
import plotly.express.trendline_functions

print("import: 'plotly.graph_objects'")
import plotly.graph_objects

print("import: '_plotly_utils'")
import _plotly_utils

print("import: '_plotly_utils.colors'")
import _plotly_utils.colors

print("import: '_plotly_future_'")
import _plotly_future_

